from AccessControl import ClassSecurityInfo

# AT & LinguaPlone support?
try:
   from Products.LinguaPlone.public import *
except ImportError:
   from Products.Archetypes.public import *
   
from Products.Archetypes.Registry import registerPropertyType, registerField

from Products.Archetypes.Field import encode
from Products.Archetypes.utils import mapply, shasattr
from Products.Archetypes.interfaces.base import IBaseUnit

from config import *
from util import fillData, checkProperties

__all__ = ["DataMergeStringField","DataMergeTextField"]


class DataMergeStringField(StringField):
   """A field that replaces fields with values"""

   __implements__ = StringField.__implements__
    
   _properties = StringField._properties.copy()
   _properties.update({
         "separators": "[]",
         "data_method": "compileMergeData",
   })

   security = ClassSecurityInfo()


   security.declarePrivate('get')
   def get(self, instance, **kwargs):
      ""     
      checkProperties(self, instance, self.data_method, self.separators)
      # optionally, set a custom context
      customcontext = kwargs.get("context", None)
      
      original = ObjectField.get(self, instance, **kwargs)
      value = fillData(original, getattr(instance,self.data_method)(customcontext), separators=self.separators)      
      return encode(value, instance, **kwargs)


   def getRaw(self, instance, **kwargs):
      ""
      value = ObjectField.get(self, instance, **kwargs)
      return encode(value, instance, **kwargs)


class DataMergeTextField(TextField):
   "allows merging data dynamically into text"

   __implements__ = TextField.__implements__

   _properties = TextField._properties.copy()
   _properties.update({
         "separators": "[]",
         "data_method": "compileMergeData",
         "edit_accessor": "getRaw"
   })

   security = ClassSecurityInfo()


   security.declarePrivate('get')
   def get(self, instance, mimetype=None, raw=False, **kwargs):
      """ If raw, return the base unit object, else return value of
      object transformed into requested mime type.

      If no requested type, then return value in default type. If raw
      format is specified, try to transform data into the default output type
      or to plain text.
      If we are unable to transform data, return an empty string. 
      """

      # make sure we're properly configured for data merge
      checkProperties(self, instance, self.data_method, self.separators)

      # optionally, set a custom context
      customcontext = kwargs.get("context", None)

      try:
         kwargs['field'] = self
         value = self.getStorage(instance).get(self.getName(), instance, **kwargs)

         if not IBaseUnit.isImplementedBy(value):
            # data merge
            value = fillData(str(value), getattr(instance, self.data_method)(customcontext), separators=self.separators)
            return encode(value, instance, **kwargs)

      except AttributeError:
         # happens if new Atts are added and not yet stored in the instance
         if not kwargs.get('_initializing_', False):
             self.set(instance, self.getDefault(instance), _initializing_=True, **kwargs)
         return self.getDefault(instance)

      if raw:
         return value

      if mimetype is None:
         mimetype = self.default_output_type or 'text/plain'

      if not shasattr(value, 'transform'): # oldBaseUnits have no transform
         # data merge
         return fillData(str(value), getattr(instance,self.data_method)(customcontext), separators=self.separators)

      data = value.transform(instance, mimetype)

      if not data and mimetype != 'text/plain':
         data = value.transform(instance, 'text/plain')

      if "text" in mimetype or "html" in mimetype or "xml" in mimetype:
         # data merge
         return fillData(data, getattr(instance,self.data_method)(customcontext), separators=self.separators)

      return data or ''



registerField(DataMergeStringField,title='Data Merge String',
   description='Used for merging data dynamically into simple strings'
)

registerField(DataMergeTextField,title='Data Merge Text',
   description='Used for merging data dynamically into text'
)

registerPropertyType('data_method', 'string')
registerPropertyType('separators', 'integer')

