import os, logging
from App.version_txt import getZopeVersion
from Products.Archetypes.public import process_types, listTypes
from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory
from AccessControl import ModuleSecurityInfo, allow_module

try:
   from Products.CMFCore.permissions import setDefaultRoles, AddPortalContent, ModifyPortalContent, setDefaultRoles
except:
   from Products.CMFCore.CMFCorePermissions import setDefaultRoles, AddPortalContent, ModifyPortalContent

import Products.CMFPlone.interfaces

#TODO: should tighten up permissions?
for func in ('validate_fields', "encode_smtp_header", "onlinehelp_availablefor"):
   ModuleSecurityInfo('Products.eCampaigning.util').declarePublic(func)

ModuleSecurityInfo('Products.eCampaigning.config').declarePublic("LAYOUTS")
ModuleSecurityInfo('Products.eCampaigning.config').declarePublic("AccessEcampaigningDataPermission")

from config import *
from content import *
import validators

registerDirectory(SKINS_DIR, GLOBALS)

# message factory setup
def MessageFactory(projectname):
   "get a i18n message factory"
   vmajor, vminor, vmicro, vstatus, vrelease = getZopeVersion()
   if (vmajor > 2) or ((vmajor, vminor) in [(2,9),(2,10)]):
      from zope.i18nmessageid import MessageFactory
   elif (vmajor, vminor) == (2,8):
      from zope.i18nmessageid import MessageIDFactory as MessageFactory
   else:
      try:
         from Products.PlacelessTranslationService.MessageID import MessageIDFactory as MessageFactory
      except:
         logging.info("[eCampaigningTool] No i18n Message Factory found -> cannot provide translations!")
         MessageFactory = lambda x: lambda x: "no message factory found"

   return MessageFactory(projectname)

ProductMessageFactory = MessageFactory(PROJECTNAME.lower())
ModuleSecurityInfo('Products.%s' % PROJECTNAME).declarePublic("ProductMessageFactory")


def initialize(context):

   content_types, constructors, ftis = process_types(listTypes(PROJECTNAME),PROJECTNAME)
   permissions = {}

   types = listTypes(config.PROJECTNAME)
   for atype in  types:
       permission = "%s: Add %s" % (config.PROJECTNAME, atype['portal_type'])
       permissions[atype['portal_type']] = permission

       # Assign default roles
       setDefaultRoles(permission, ('Owner','Manager'))

   allTypes = zip(content_types, constructors)
   for atype, constructor in allTypes:
      kind = "%s: %s" % (PROJECTNAME, atype.archetype_name)
      utils.ContentInit(
            kind,
            content_types      = (atype,),
            permission         = permissions[atype.portal_type],
            extra_constructors = (constructor,),
            fti                = ftis,
      ).initialize(context)


   # register the Action Record
   import content.record
   context.registerClass(
        content.record.ActionRecord,
        constructors = (content.record.addRecord, content.record.addRecordForm),
        icon = 'www/actionrecord.png'
   )

   # Custom permissions default roles
   setDefaultRoles(AccessEcampaigningDataPermission,('Owner','Manager'))


   # Register generic setup profile
   if HAVE_GENERIC_SETUP:
      from Products.GenericSetup import EXTENSION
      from Products.GenericSetup import profile_registry
      profile_registry.registerProfile(
         name='default',
         title='eCampaigning Tool',
         description='Profile for eCampaigning Tool',
         path='profiles/default',
         product='eCampaigning',
         profile_type=EXTENSION,
         for_=Products.CMFPlone.interfaces.IPloneSiteRoot
      )
