import string, os
#from csv import DictReader
try:
   from Products.CMFCore.permissions import AddPortalContent
except:
   from Products.CMFCore.CMFCorePermissions import AddPortalContent

from Products.Archetypes.public import DisplayList
from Products.CMFCore.utils import getToolByName
from Globals import package_home

GLOBALS = globals()
PACKAGE_HOME = package_home(globals())
PROJECTNAME = "eCampaigning"

# Check if we have GenericSetup
try:
    from Products.GenericSetup import EXTENSION
    HAVE_GENERIC_SETUP = True
except:
    HAVE_GENERIC_SETUP = False


# name of custom script used for populating data merge namespace
CUSTOM_DATAMERGE_SCRIPT = "getMergeData"

# Don't show these in the navigation tree portlet
NOTINNAVTREE = (
   "Layout Element",
   "Organization Target",
   "Person Target",
   "Thank You Email Template",
   "Thank You Page Template",
   "Field Block",
   "HTML Block",
   "Action Step",
   "Action Message Template",
   "Data Merge Content"
)
## PERMISSIONS ##################
ADD_CONTENT_PERMISSION = AddPortalContent
AccessEcampaigningDataPermission = "Access ecampaigning data"


## SKIN INSTALL ##################
SKINS_DIR = 'skins'
BASESKIN = "Plone Tableless"

# The existing layers contained in the BASESKIN will be copied to these new skins.
#SKINS = {"Public": "overlay, images, scripts"}

# The layers that will be installed to all skins
COMMONLAYERS = ('ecampaigning_templates','ecampaigning_signup','ecampaigning_images','ecampaigning_scripts','ecampaigning_remoting')


## ON-LINE INTEGRATION ###############
URL_UPDATECHECK = "http://updatecheck.fairsay.com/check/ectool"
URL_ACTIONREGISTRY = "http://actionregistry.fairsay.com/register"
URL_TOOLHELP = "http://toolhelp.fairsay.com/ectool"
ONLINEHELP_AVAILABLEFOR = (
   ## Campaign Action and Action Edition ##
   "title","name","description","subject","coverage","publisher","language","effectiveDate","expirationDate",

   ## Campaign Action ##
   "action_count_threshold","onsite_action_goal",

   ## Action Edition ##
   "actiontype","parentaction","targets","messagetemplate","action_message_modifiable","action_message_modify_as_plaintext",
   "thankyouemailname","thankyouemailfrom","thankyouemail","thankyouemail_includemessage","thankyoupage",
   "edition_count_threshold","onsite_edition_goal",

   ## Templates ##
   "messagetitle",

   ## Targets ##
   "emailaddress","givenname","familyname",

   ## Action Step ##
   "step_layout","step_validators","step_tasks",
   "step_submitlabel","step_custom_validator","step_custom_task",

   ## Layout Element ##
   "datasource","layoutelement",

   ## HTML and Field Blocks ##
   "displaycondition","fieldname","customaccessor"
)

# fields checked upon publication
REGISTRATION_REQUIREDFIELDS = (
   "title",
   "description",
   "subject",
   "coverage",
   "publisher",
   "language",
   "effectiveDate",
   "expirationDate",
)

## PROPERTY SHEET #####################
PROPERTYSHEET = "ecampaigning_properties"

PROPERTIES = {

   # Predefined Action Types
   # Note: these also need coresponding entries in 
   #    A. configure_xxx files in /skins/ecampaigning_scripts/ and 
   #    B. Layouts for each layout element (below)
   #    C. MSGIDs in the pot and po files.
   
   "action_types":(
      ("simple_petition", "Simple e-Petition"),
      ("simple_letter", "Simple e-Letter"),
      ("simple_emailsubscription", "Simple Email Subscription"),
      ("multistep_petition01", "Multi-step e-Petition > Address"),
      ("multistep_letter01", "Multi-step e-Letter > Address"),
      ("multistep_emailsubscription01", "Multi-step Email Subscription > Personal Details"),
      ("smartstep_petition01", "Smart-step e-Petition > Optin/Address"),
      ("smartstep_letter01", "Smart-step e-Letter > Optin/Address"),
      ("custom", "Custom action"),
   ),
   
   # map titles to validator python scripts

   "step_validator_vocabulary":(
      ("perform basic form validation", "stepvalidate_standard"),
   ), 

   # map titles to controller python scripts

   "step_task_vocabulary":(
      ("Store form contents", "steptask_storeform"),
      ("Store tracking information", "steptask_storetrackinginfo"),
      ("Update action listing", "steptask_to_recentlisting"),      
      ("Send thanks email", "steptask_sendthankyouemail"),
      ("Send email to targets", "steptask_emailtargets"),      
   ),

   # map to template macro names and to the required layout element definitions
   
   "layout_vocabulary":(
      ("One-column action page with header & footer", "onecolumn:header+content+footer"),
      ("Two-column action page with header & footer", "twocolumn:header+left+right+footer"),
      ("One-column extra data collection page", "addr_onecol:addr_page_header+addr_page_content+addr_page_footer"),
      ("Thank-you page with header, content area & footer", "thankyou:header+content+footer"),
   ),

   "element_vocabulary":(
      ("header","Page header"),
      ("content","Page content"),
      ("footer","Page footer"),
      ("left","Left column"),
      ("middle","Middle column)"),
      ("right","Right column"),
      ("addr_page_header","Header (Extra data)"),
      ("addr_page_content","Content (Extra data)"),
      ("addr_page_middle","Middle (Extra data)"),
      ("addr_page_footer","Footer (Extra data)"),
      ("smart_header","Header (Smart data)"),
      ("smart_content","Content (Smart data)"),
      ("smart_footer","Footer (Smart data)"),
      ("disabled", "Disabled Element"),
   ),

   "ActionEdition_signup_alternatives":(
      ("Single column action page","onecolumn"),
      ("Single column action page + a page for extra data collection","onecolumn+addr_onecol"),
      ("Dual column action page","twocolumn"),
      ("Dual column action page + a page for extra data collection","twocolumn+addr_onecol"),
   ),
}


## STEP LAYOUTS  ##########################
HB = "HTML Block"
FB = "Field Block"
HB_DATA = ""
HB_DEFAULT = (HB, HB_DATA)

# Styled Progress bar (Plone, Apple, Vista or Ubuntu) #
# Change progress-plone to the style you want from step_styles.css #
HB_PROGRESSBAR_HTML = """
<table width="100%">
  <tr>
    <td>0</td>
    <td align="right"><span omit-tag="" i18n:translate="msg_action_goal">Goal</a>: [onsite_edition_goal]</td>
  </tr>
  <tr>
    <td colspan="2">
      <div id="progress-plone">
        <div class="fill" style="width: [onsite_edition_progress]%;"></div>
      </div>
      <div align="center" class="discreet">[onsite_edition_count] <span omit-tag="" i18n:translate="msg_progressbar_subtext">action supporters</span></div>
    </td>
  </tr>
</table>
"""

HB_PROGRESS_CONDITION = "python: edition.onsite_edition_goal > 0"

HB_PROGRESSBAR = (HB,HB_PROGRESSBAR_HTML,HB_PROGRESS_CONDITION)

ON_OPTIN_Y = "record/receive_updates"
ON_OPTIN_N = "not:record/receive_updates"
ON_OPTIN_H = """<p>You have chosen not to be kept informed of how the campaign progresses and we'll respect this choice.  However if you change your mind simply check the box below and press 'update' and we'll keep you informed.</p>"""

LAYOUTS = {

   "basic_signup": (
      ("header", (HB_DEFAULT,)),
      ("left", (HB_PROGRESSBAR,(FB,"messagesubject"),(FB,"message"))),
      ("right", ((FB,"givenname"), (FB,"familyname"), (FB,"emailaddress"), (FB,"receive_updates"), (FB,"country"),  (FB,"anonymous"), (FB,"name_df"))),
      ("footer", (HB_DEFAULT,))
   ),

   "email_signup": (
      ("header", (HB_DEFAULT,)),
      ("content", ((FB,"emailaddress"),(FB,"name_df"))),
      ("footer", (HB_DEFAULT,))
   ),

   "address_collection": (
      ("addr_page_header", (HB_DEFAULT,)),
      ("addr_page_content", ((FB,"streetaddress"),(FB,"postalcode"),(FB,"city"))),
      ("addr_page_footer", (HB_DEFAULT,)),
   ),

   "name_address_collection": (
      ("addr_page_header", (HB_DEFAULT,)),
      ("addr_page_content",((FB,"givenname"),(FB,"familyname"),(FB,"streetaddress"),(FB,"postalcode"),(FB,"city"),(FB,"country"))),
      ("addr_page_footer", (HB_DEFAULT,)),
   ),

   "smart_optin_address": (
      ("smart_header", (HB_DEFAULT,)),
      ("smart_content",((FB,"getThanksPageSubject",ON_OPTIN_N),(HB,ON_OPTIN_H,ON_OPTIN_N),(FB,"receive_updates",ON_OPTIN_N),(FB,"getThanksPageText",ON_OPTIN_N),(FB,"streetaddress",ON_OPTIN_Y),(FB,"postalcode",ON_OPTIN_Y),(FB,"city",ON_OPTIN_Y))),
      ("smart_footer", (HB_DEFAULT,)),
   ),

   "thanks_page": (
      ("header", (HB_DEFAULT,)),
      ("content", ((FB,"getThanksPageSubject"),(FB,"getThanksPageText"))),
      ("footer", (HB_DEFAULT,))
   )

}


## COUNTRY LISTS #########################
COUNTRYLISTS = {}

fdir = PACKAGE_HOME + os.sep + "i18n"
filenames = [n for n in os.listdir(fdir) if n[:10]=="countries_" and n[-4:]==".txt"]

for fname in filenames:
   countrylist = []
   language = fname[-6:-4]
   for l in open(fdir + os.sep + fname):
      l=l.strip('\n')
      countrylist.append((l.split('|')))
   COUNTRYLISTS[language] = tuple(countrylist)

   
## PRECREATED CONTENT (upon edition creation) ######################

PRECREATE = (
   "Person Target",
   "Action Message Template",
   "Thank You Page Template",
   "Thank You Email Template"
)
