from StringIO import StringIO
import logging

# Zope
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view, access_contents_information

#CMF
try:
   from Products.CMFCore.permissions import View, ModifyPortalContent
except:
   from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from Products.CMFCore.utils import getToolByName


# AT & LinguaPlone support?
try:
   from Products.LinguaPlone.public import *
except ImportError:
   from Products.Archetypes.public import *

from Products.Archetypes.interfaces.base import *

#ATCT
try:
   from Products.ATContentTypes.content.folder import ATFolder #,ATBTreeFolder
   from Products.ATContentTypes.content.schemata import finalizeATCTSchema, marshall_register
# old ATCT
except ImportError:
   from Products.ATContentTypes.types.ATFolder import ATFolder
   #from Products.ATContentTypes.types.ATFolder import ATBTreeFolder
   finalizeATCTSchema = lambda x: None

# Third-party
from Products.ATSchemaEditorNG.ParentManagedSchema import ParentManagedSchema
from Products.ATSchemaEditorNG.SchemaEditor import SchemaEditor
from Products.ATReferenceBrowserWidget import ATReferenceBrowserWidget

# Package
from Products.eCampaigning.config import AccessEcampaigningDataPermission #, VERSION
from Products.eCampaigning import util

from record import *
from schemas import ActionSchema, ActionEditionSchema, ActionRecordSchema

__all__ = ["CampaignAction","CampaignActionEdition"]

# Campaign Action
class CampaignAction(ATFolder):
   "campaign action content"

   typeDescription= 'Parent container for different action related elements.'
   typeDescMsgId  = 'description_edit_campaignaction'

   security = ClassSecurityInfo()

   meta_type = portal_type = archetype_name = "Campaign Action"
   global_allow = 1
   content_icon = "campaignaction.png"

   _at_rename_after_creation = True

   allowed_content_types = [
            "Person Target",
            "Organization Target",
            "Action Edition",
            "Thank You Page Template",
            "Thank You Email Template",
            "Action Message Template",
            "I18NFolder",
            "Image",
            "Document",
            "File",
            "ATImage",
            "ATDocument",
            "ATFile",
            "Wiki Page",
            "ZWiki Page"
   ]

   _s_fields = tuple([f.copy() for f in ATFolder.schema.fields()]) + ActionSchema
   schema = ManagedSchema(_s_fields)
   
   # Further schema customization
   for f in schema.fields():
      if f.schemata == "default":
         schema.changeSchemataForField(f.getName(), "general")
         
   schema.moveField('id',pos="top")
   schema.moveField('title', after="id")
   schema.moveField("name", after="title")
   schema.moveField('description', after="name")

   schema["description"].widget.label = "Public Description"
   schema["description"].widget.label_msgid = "label_action_description"
   schema["description"].widget.description = "A short summary of the Action Edition. This is public and used to help promote it via the Action Registry."
   schema["description"].widget.description_msgid = "help_action_description"
   schema["description"].widget.i18n_domain = "ecampaigning"

   schema["effectiveDate"].schemata = "general"
   schema["effectiveDate"].widget.description = "The date and time the Action launches. This helps promote it via the Action Registry."
   schema["effectiveDate"].widget.description_msgid = "help_action_effectiveDate"
   schema["effectiveDate"].widget.i18n_domain = "ecampaigning"
   
   schema["expirationDate"].schemata = "general"
   schema["expirationDate"].widget.description = "The date and time the Action closes. This helps ensure (via the Action Registry) that it isn't promoted after bring closed."
   schema["expirationDate"].widget.description_msgid = "help_action_expirationDate"
   schema["expirationDate"].widget.i18n_domain = "ecampaigning"
      
   schema.changeSchemataForField('allowDiscussion', 'general') 
   schema.moveField('allowDiscussion', pos="bottom")

   schema["subject"].schemata = "advanced"
   schema["subject"].widget.description = "Select or add keywords that reflect the key concepts of the Action. Used to help promote the Action via the Action Registry."
   schema["subject"].widget.description_msgid = "help_action_subject"
   schema["subject"].widget.i18n_domain = "ecampaigning"

   schema.moveSchemata("general", -1)
   schema.moveSchemata("advanced", 1)

   marshall_register(schema)

   actions = ATFolder.actions + (

        { 'id': 'getdata',
          'name': 'Get data',
          'action': 'string:${object_url}/datafile',
          'permissions': (ModifyPortalContent,),
          'category': 'object',
          'visible':True
        },

        { 'id': 'recentactions',
          'name': 'Recent actions',
          'action': 'string:$object_url/actions_tabular_view',
          'permissions': (View,),
          'category': 'object'
        },

        { 'id': 'update',
          'name': 'Check for updates',
          'action': 'string:${object_url}/updates_check',
          'permissions': (ModifyPortalContent,),
          'category': 'object',
          'visible':True
        }
   )

   default_view = "campaign_action_overview"


   def getEditions(self, contained=True, referred=True):
      "get edition objects referring to this (parent) action"
      editions = []
      if referred:
         editions.extend(self.getBackReferences(relationship="isPartOfAction"))
      if contained:
         mypath = "/".join(self.getPhysicalPath())
         catalog = getToolByName(self,"portal_catalog")
         editions.extend([e.getObject() for e in catalog(portal_type=["Action Edition",], path=mypath) if e.getObject() not in editions])
      return editions

      
   def getRecordCount(self,rtypes=("Action Record",),ctypes=("Action Edition",)):
      "common for actions & editions"
      count = 0
      return sum([e.getRecordCount() for e in self.getEditions()])


   def getRecords(self,rtypes=("Action Record",),ctypes=("Action Edition",)):
      "used by dataexport - get all records from all editions"
      for edition in self.getEditions():
         # use the generator
         for r in edition.getRecords(rtypes):
            yield r

            
   def getRecentRecords(self, **kwargs):
      """get recent action records from all editions,
         optionally passing sort_on (the index name) & sort_order ('reverse'),
         and/or any other query args
      """

      recordcatalog_id = getToolByName(self, "portal_properties")["ecampaigning_properties"].actionrecord_catalog
      record_catalog = getattr(self, recordcatalog_id)

      sort_on = kwargs.get("sort_on", "datetime")

      # we don't want to pass sort param to catalog query
      # since we're sorting the total manually
      try:
         del kwargs["sort_on"]
      except:
         pass

      sort_order = kwargs.get("sort_order", "reverse")
      kwargs["anonymous"] = kwargs.get("anonymous", False)
     
      ascending = lambda x, y, sort_on: getattr(x, sort_on) > getattr(y, sort_on) and 1
      descending = lambda x, y, sort_on: getattr(x, sort_on) < getattr(y, sort_on) and -1
      sorter = lambda x, y: ascending(x, y, sort_on) or descending(x, y, sort_on)

      containerpaths = [edition.getRecordPath() for edition in self.getEditions()]
      records = []
      
      for cpath in containerpaths:
         records.extend(record_catalog(path=cpath, **kwargs))

      records.sort(cmp=sorter)
      if sort_order == "reverse":
         records.reverse()

      return records

      
registerType(CampaignAction)


# Action Edition
class CampaignActionEdition(ATFolder, SchemaEditor):
   "base class for campaign action editions"

   typeDescription= 'Configurable e-action containing steps and other action elements.'
   typeDescMsgId  = 'description_edit_campaignactionedition'

   security = ClassSecurityInfo()
   action_record = "Action Record"
   global_allow = 1
   filter_content_types = 1
   meta_type = portal_type = archetype_name = "Action Edition"
   content_icon = "actionedition.png"
   default_view = "takeaction"

   # support variable for detecting type changes
   _previous_actiontype = None
   
   __implements__ = ATFolder.__implements__ + SchemaEditor.__implements__
   
   allowed_content_types = [
         "I18NFolder",
         "Person Target",
         "Organization Target",
         "Thank You Page Template",
         "Thank You Email Template",
         "Action Message Template",
         "Action Step",
         "Large Plone Folder",
         "I18NFolder",
         "Image",
         "Document",
         "File",
         "ATImage",
         "ATDocument",
         "ATFile",
         "Wiki Page",
         "ZWiki Page"
   ]
   _s_fields = tuple([f.copy() for f in ATFolder.schema.fields()]) + ActionEditionSchema
   schema = ManagedSchema(_s_fields)

   # Further schema customization
   for f in schema.fields():
      if f.schemata == "default":
         schema.changeSchemataForField(f.getName(), "general")
         
   schema.moveField('id',pos="top")   
   schema.moveField('title', after="id")
   schema.moveField("name", after="title")
   schema.moveField('description', after="name")

   schema["description"].widget.label = "Public Description"
   schema["description"].widget.label_msgid = "label_action_description"
   schema["description"].widget.description = "A short summary of the Action Edition. This is public and used to help promote it via the Action Registry."
   schema["description"].widget.description_msgid = "help_action_description"
   schema["description"].widget.i18n_domain = "ecampaigning"

   schema["effectiveDate"].schemata = "general"
   schema["effectiveDate"].widget.description = "The date and time the Action Edition launches. This helps promote it via the Action Registry."
   schema["effectiveDate"].widget.description_msgid = "help_action_effectiveDate"
   schema["effectiveDate"].widget.i18n_domain = "ecampaigning"
   
   schema["expirationDate"].schemata = "general"
   schema["expirationDate"].widget.description = "The date and time the Action Edition closes. This helps ensure (via the Action Registry) that it isn't promoted after bring closed."
   schema["expirationDate"].widget.description_msgid = "help_action_expirationDate"
   schema["expirationDate"].widget.i18n_domain = "ecampaigning"
   
   schema["subject"].schemata = "advanced"
   schema["subject"].widget.description = "Select or add keywords that reflect the key concepts of the Action Edition. Used to help promote the Action Edition via the Action Registry."
   schema["subject"].widget.description_msgid = "help_action_subject"
   schema["subject"].widget.i18n_domain = "ecampaigning"
   schema.moveField('subject', after="edition_count_threshold")

   schema["language"].schemata = "advanced"
   schema['language'].default_method = "getDefaultLanguage"
   schema["language"].widget.description = "Ensures the Action Edition displays properly on multi-lingual sites and is properly promoted via the Action Registry."
   schema["language"].widget.description_msgid="help_action_language"
   schema["language"].widget.i18n_domain="ecampaigning"
   schema.moveField('language', pos="bottom")
   
   schema.moveField('allowDiscussion', pos="bottom")
   
   schema.moveSchemata("general", -1)
   schema.moveSchemata("advanced", 1)
   schema.moveSchemata("advanced", 1)
   
   marshall_register(schema)

   actions = ATFolder.actions + (


        { 'id': 'overview',
          'name': 'Overview',
          'action': "string:${object_url}/base_view",
          'permissions': (View,),
          'category': 'object'
        },
        
        { 'id': 'editschemas',
          'name': 'Edit Schemas',
          'action': 'string:$object_url/atse_editor',
          'permissions': (ModifyPortalContent,),
          'category': 'object'
        },

        { 'id': 'recentactions',
          'name': 'Recent actions',
          'action': 'string:$object_url/actions_tabular_view',
          'permissions': (View,),
          'category': 'object'
        },

        { 'id': 'getdata',
          'name': 'Get data',
          'action': 'string:${object_url}/records/datafile',
          'permissions': (ModifyPortalContent,),
          'category': 'object',
          'visible':True
        }
   )


   if not hasattr(ATFolder, "__browser_default__"):
      from Products.PageTemplates.PageTemplateFile import PageTemplateFile
      from Products.eCampaigning.config import PACKAGE_HOME
      import os
      ptpth = os.sep.join((PACKAGE_HOME,)+ tuple("skins/ecampaigning_templates/eaction_edition_view.pt".split('/')))
      index_html = PageTemplateFile(ptpth, globals())


   def normalizeName(s):
      ""
      return s.replace(' ','_').lower()


   #def canSetDefaultPage(self):
   #   "no need"
   #   return False
      

   def configureAction(self, preset_id):
      "configure edition using a preset script"
      
      self._previous_actiontype = self.getActiontype()
               
      if preset_id == "custom":
         return

      configscript = getattr(self, "configure_" + preset_id, None)
      if configscript:
         step_ids = self.objectIds("Action Step")
         self.manage_delObjects(ids=step_ids)
         configscript()
      else:
         self.restrictedTraverse("view?portal_status_message=no configuration script for %s" % preset_id)

         
   def at_post_create_script(self):
      "add record store, register & customize schema, configure action, hookup content & setup permissions"
      
      self.manage_addProduct["BTreeFolder2"].manage_addBTreeFolder(id="records", title="Action Records")      
      record_store = getattr(self, "records")
      record_store.manage_permission(view, roles=["Owner", "Manager"])
      record_store.manage_permission(access_contents_information, roles=["Owner", "Manager"])

      # do not show default or metadata fieldset
      #self.atse_registerObject(ActionRecord, ('default', 'metadata','message'))
      schema = Schema(ActionRecordSchema)
      self.atse_registerSchema("Action Record",  schema, domain='ecampaigning')

      actiontype = self.getActiontype()      
      if actiontype != self._previous_actiontype:
         self.configureAction(actiontype)
         
      self.manage_permission("ATSE: Manage schema", ('Owner',), acquire=1, REQUEST=self.REQUEST)
  
      
   def at_post_edit_script(self):
      "configure edition based on new action type selection"
      
      actiontype = self.getActiontype()
      if actiontype != self._previous_actiontype:
         self.configureAction(actiontype)

   
   def createDefaultContent(self):
      "create message, target & thanks content"
      from config import PRECREATE
      for t in PRECREATE:
         default_id = "default-" + t.lower().replace(' ', '-')
         s = self.createObject(type_name=t, id=default_id)
         s.context.setTitle("Example " + t)
         s.context.setDescription("This is placeholder content awaiting customization")
      
      #  by default, set target to user 
      target = getattr(self, "default-person-target")
      userid = context.portal_membership.getAuthenticatedUser()
      useremail = self.portal_membership.getMemberById(userid).getProperty("email")
      target.setEmailaddress(useremail)
      target.setGivenname("Test")
      target.setFamilyname("Target")
   
   
   def hookupDefaultContent(self):
      "attempt to hook up templates, targets etc."
      try:
         self.setMessagetemplate(getattr(self,"default-action-message-template").UID())
      except:
         pass

      try:
         self.setThankyouemail(getattr(self,"default-thank-you-email-template").UID())
      except:
         pass

      try:
         self.setThankyoupage(getattr(self, "default-thank-you-page-template").UID())
      except:
         pass

      try:
         self.setTargets((getattr(self,"default-person-target").UID(),))
      except:
         pass

      # TODO: remove if not needed
      #if self.aq_parent.portal_type == "Campaign Action":
      #   self.setParentaction(self.aq_parent.UID())
         
      
   def getRecordFieldNames(context, managed_only=True):
      "ALL names"
      schema = context.atse_getSchemaById("Action Record")
      all_fields = schema.keys()
      unmanaged_fields = context.atse_getUnmanagedFieldNames("Action Record")
      if managed_only:
         return [fn for fn in all_fields if fn not in unmanaged_fields]
      return all
    
   def schemasVocabulary(self):
      return tuple([(s,s) for s in self._schemas.keys()])

   def schemataVocabulary(self, schemaname):
      return tuple([(s,s) for s in self._schemas[schemaname].keys()])

   def fieldVocabulary(self,schemaname):
      return tuple([(f,f) for f in self._schemas[schemaname].keys()])

   # TODO: remove if moving to util causes no ill effects   
   #def getElementsForLayout(self, layoutname):
   #   "get a list of elements that are used in a given layout"
   #   props = getToolByName(self,"portal_properties")["ecampaigning_properties"].layout_vocabulary
   #   data = [l for l in props if layoutname in l]
   #   if data:
   #      return data[0].strip().split('|')[1].split(':')[1].split('+')
   #   return ()
      

   def layoutVocabulary(self):
      "get the layout vocabulary"
      props = getToolByName(self,"portal_properties")["ecampaigning_properties"].layout_vocabulary
      return tuple([(l.split('|')[1].split(':')[0], l.split('|')[0]) for l in props])
      

   def getVocabularyItem(self, vocabulary_id, item_id):
      "get a single item"
      props = getattr(getToolByName(self,"portal_properties")["ecampaigning_properties"], vocabulary_id)
      item = [l.strip().split('|') for l in props if item_id in l][0]
      return tuple(item)


   def validatorVocabulary(self):
      "get the validator vocabulary for selection upon step creation"
      props = getToolByName(self,"portal_properties")["ecampaigning_properties"].step_validator_vocabulary
      v = [l.strip().split('|') for l in props]
      return tuple([(id, label) for label, id in v])


   def taskVocabulary(self):
      "get the task vocabulary for selection upon step creation"
      props = getToolByName(self,"portal_properties")["ecampaigning_properties"].step_task_vocabulary
      v = [l.strip().split('|') for l in props]
      return tuple([(id, label) for label, id in v])

     
   def getLayoutDefinition(self,layoutname):
      "middle/left/right etc"

      defined_layouts = dict([(o.getLayoutelement(),o.getId()) for o in self.objectValues("Layout Element Definition")])

      if layoutname in defined_layouts:
         return getattr(self,defined_layouts[layoutname])
      else:
         return None

         
   # TODO: convert all direct references to use getRecordContainer instead

   security.declareProtected(AccessEcampaigningDataPermission,"getRecordContainer")
   def getRecordContainer(self):
      "get the container object"
      return self.records


   def getRecordPath(self):
      "get the folder physical path"
      return '/'.join(self.getRecordContainer().getPhysicalPath())


   def getRecordCount(self,rtypes=("Action Record",)):
      "common for actions & editions"
      return len(self.getRecordContainer()) #(rtypes=rtypes))


   security.declareProtected(AccessEcampaigningDataPermission,"getRecords")
   def getRecords(self,rtypes=("Action Record",)):
      "used by data export, common for actions & editions"
      try:
         return self.getRecordContainer().objectValues(rtypes)
      except Exception, e:
         logging.getLogger("Plone").error("could not retrieve records from record container: %s" % e)
         return []


   def getRecentRecords(self, **kwargs):
      """get recent action records, optionally passing sort_on (the index name) & sort_order ('reverse')"""
      kwargs["sort_on"] = kwargs.get("sort_on", "getDate")
      kwargs["sort_order"] = kwargs.get("sort_order", "reverse")
      kwargs["anonymous"] = kwargs.get("anonymous", False)

      parent = self.getParentaction()
      if parent:
         # the parent sorts on metadata column
         kwargs["sort_on"] = kwargs.get("parent_sort_on", "datetime")
         return parent.getRecentRecords(**kwargs)
      else:
         kwargs["path"] = kwargs.get("path", self.getRecordPath())
         recordcatalog_id = getToolByName(self, "portal_properties")["ecampaigning_properties"].actionrecord_catalog
         recordcatalog = getattr(self, recordcatalog_id)
         return recordcatalog(**kwargs)


   def getDefaultLanguage(self):
      "if not set, get the portal language"
      return getToolByName(self,"portal_properties")["site_properties"].default_language


   def getActionTypesVocabulary(self):
      "get a DisplayList for selecting an Action Type"
      atypedata = getToolByName(self,"portal_properties")["ecampaigning_properties"].action_types
      return DisplayList([d.split('|') for d in atypedata])

   
   #TODO: remove old getActionTypesVocabulary
   #def getActionTypesVocabulary(self):
   #   "get a DisplayList for selecting an Action Type"
   #   types = zip(map(lambda x: x.lower().replace("-","_"), ACTIONTYPES), ACTIONTYPES)
   #   return DisplayList(types)

    
   def defaultThankyouFromAddress(self):
      "AT field default_method"
      return self.portal_url.getPortalObject().getProperty(id="email_from_address")
      
   def defaultThankyouFromName(self):
      "AT field default_method"
      return self.portal_url.getPortalObject().getProperty(id="email_from_name")

   
   # SHORTCUTS TO REFERENCED CONTENT
   
   def getThanksPageText(self):
      "get text content of the thank-you page template"
      try:
         return self.getThankyoupage().getText()
      except:
         return ""

   def getThanksPageSubject(self):
      "get subject of the thank-you page template"
      try:
         return self.getThankyoupage().getMessagetitle()
      except:
         return ""
         
   def getThanksEmailText(self, **kwargs):
      "get text content of the thank-you email template"
      try:
         template = self.getThankyouemail().aq_inner
         accessor = template.getField('text').accessor
         return getattr(template, accessor)(**kwargs)
      except:
         return ""
   
   def getMessageText(self, mimetype=None):
      "get message template text content"         
      try:
         template = self.getMessagetemplate()
         original_mimetype = template.getContentType()
         mimetype = mimetype or original_mimetype
         return template.getText(mimetype=mimetype)
      except:
         return ""

   def getMessageSubject(self):
      "get message template subject"
      try:
         return self.getMessagetemplate().getMessagetitle()
      except:
         pass

   def addStep(self, id, title):
      "add an action step"
      new_id = self.invokeFactory(type_name="Action Step", id=id)
      id = new_id or id
      step = getattr(self, id)
      step.setTitle(title)
      step.reindexObject(idxs=["Title"])
      return step
   

   def getFirstStepId(self):
      "return the first action step"
      steps = [(self.getObjectPosition(id), id) for id in self.contentIds("Action Step")]
      steps.sort()
      return steps[0][1]


   def getCurrentStepNumber(self):
      current = self.REQUEST["SESSION"]["stepid"]
      data = [self.getObjectPosition(id) for id in self.contentIds("Action Step")]
      data.sort()
      return data.index(self.getObjectPosition(current)) + 1


   def Publisher(self):
      "dc metadata - also used for owner"
      return self.schema["publisher"].get(self)
      
   def getClassifiers(self):
      "use the keywords for action classification"
      return '+'.join(self.Subject())

   def isPromotable(self):
      "return true if the edition passes promotability check"
      required_fields = getToolByName(self,"portal_properties")["ecampaigning_properties"].registration_requiredfields
      for f in required_fields:
         if not self.getField(f).get(self):
            return False
      return True
            
registerType(CampaignActionEdition)
