from AccessControl import ClassSecurityInfo
from Acquisition import ImplicitAcquisitionWrapper

from Products.PageTemplates.Expressions import getEngine

try:
   from Products.CMFCore.permissions import View, ModifyPortalContent
except:
   from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from Products.CMFCore.utils import getToolByName

# AT & LinguaPlone support?
try:
   from Products.LinguaPlone.public import *
except ImportError:
   from Products.Archetypes.public import *

#ATCT
try:
   from Products.ATContentTypes.content.document import ATDocument
   from Products.ATContentTypes.content.folder import ATFolder   
   from Products.ATContentTypes.content.schemata import finalizeATCTSchema
   from Products.ATContentTypes.content.base import ATCTContent
# old ATCT
except ImportError:
   from Products.ATContentTypes.types.ATDocument import ATDocument
   from Products.ATContentTypes.ATContentType import ATCTContent
   finalizeATCTSchema = lambda x: None

from Products.ATDataMerge.content import DataMergeStringField, DataMergeTextField
from Products.eCampaigning import util as util

__all__ = ["HTMLBlock", "FieldBlock"]


class ConditionalBlock:
   "base"

   schema = Schema((
               StringField("displaycondition",
                  required=False,
                  searchable=False,
                  validators = ("talesvalidator",),
                  widget=StringWidget(
                     label = "Display condition",
                     label_msgid = "label_blockdisplaycondition",
                     description = "A TALES expression which, when True, results in the block being displayed.",
                     description_msgid = "help_blockdisplaycondition",
                     i18n_domain = "ecampaigning"
                  )
               ),
   ))

   def isVisible(self):
      "should this block be shown?"
      context = util.getActionExpressionContext(self)
      expressionstring = self.getField("displaycondition").getRaw(self)
      try:
         result = util.evaluateTales(context, expressionstring)
      except:
         result = "error"
      return bool(result)


class HTMLBlock(ATDocument, ConditionalBlock):
   "part of a layout element such as header or footer, that contains HTML"

   typeDescription= 'Defines the HTML contents of a field belonging to a layout element, such as a header or footer.'
   typeDescMsgId  = 'description_edit_htmlblock'

   global_allow=0
   meta_type = portal_type = archetype_name = "HTML Block"
   content_icon = "html.png"
   allow_discussion = 0
   
   schema = ATDocument.schema.copy() + ConditionalBlock.schema.copy() + Schema((

      DataMergeTextField("text",
        required=True,
        searchable=True,
        primary=True,
        default_content_type = 'text/html',
        default_output_type = 'text/x-html-captioned',
        allowable_content_types = ("text/plain", "text/html"),
        widget=RichWidget(
           label = "HTML content",
           label_msgid = "label_htmlblock_content",
           description = "The HTML content to be show on the form",
           description_msgid = "help_htmlblock_content",
           rows = 25,
           i18n_domain = "ecampaigning",
           allow_file_upload = True
        )
      ),
   ))

   # TODO: go through the templates and replace references to content field with refs to text field"
   #schema["content"].widget.visible = -1
   schema["description"].schemata = "metadata"
   finalizeATCTSchema(schema)


   # ATDataMerge support   
   def compileMergeData(self, customcontext):
      "provide default fields & values for data merge, plus a hook for customizations ('getMergeData')"
      context = customcontext or self

      # try & get the record & set the context
      try:
         context = self.restrictedTraverse(self.REQUEST.SESSION["record_path"])
      except:
         pass

      return util.getDefaultMergeData(context)

registerType(HTMLBlock)


class FieldBlock(ATCTContent, ConditionalBlock):
   "part of a layout element such as header or footer, containing the value of a field"

   typeDescription= 'Defines the attribute whose value is used as the contents of a field belonging to a layout element, such as a header or footer.'
   typeDescMsgId  = 'description_edit_fieldblock'

   global_allow=0
   meta_type = portal_type = archetype_name = "Field Block"
   content_icon = "field.png"
   allow_discussion = 0
   
   schema = BaseContent.schema.copy() + Schema((

      ComputedField('title',
                expression = "context._fieldName()",
                accessor='Title',
      ),

      StringField('fieldname',
         vocabulary = "_fieldVocab",
         widget=SelectionWidget(label="Data Source Field",
            label_msgid="label_fieldblock_fieldname",
            description="The field name of a form field you want to be displayed in the Layout Element (defined via the 'edit schema' tab).",
            description_msgid="help_fieldblock_fieldname",
            i18n_domain="ecampaigning"
         ),
      ),
      
      StringField('customaccessor',
         widget=StringWidget(label="Custom Data Accessor",
            description="Alternative custom data access method to use; if given, is used instead of the field selected above",
            label_msgid="label_fieldblock_customaccessor",
            description_msgid="help_fieldblock_customaccessor",
            i18n_domain="ecampaigning"
         ),
      ),

   )) + ConditionalBlock.schema.copy()

   schema["description"].schemata = "metadata"

   # TODO: switch to using custom accessors in field block as well (change templates correspondingly)
   """
   def getFieldname(self):
      "return result of evaluating the fieldname in the context of the record (fall back to edition)"
   
   def getCustomaccessor(self):
      "return result of evaluating the accessor in the context of the record (fall back to edition)"
   """

   def _fieldVocab(self):
      return self.fieldVocabulary(self.getDatasource())

   def _fieldName(self):
      return self.getFieldname()


registerType(FieldBlock)
