from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
from Acquisition import ImplicitAcquisitionWrapper

try:
   from Products.CMFCore.permissions import View, ModifyPortalContent
except:
   from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

# AT & LinguaPlone support?
try:
   from Products.LinguaPlone.public import *
except ImportError:
   from Products.Archetypes.public import *

#ATCT
try:
   from Products.ATContentTypes.content.folder import ATFolder   
   from Products.ATContentTypes.content.schemata import finalizeATCTSchema
# old ATCT
except ImportError:
   from Products.ATContentTypes.types.ATFolder import ATFolder
   finalizeATCTSchema = lambda x: None


__all__ = ["LayoutElement"]

   
class LayoutElement(ATFolder):
   "Defines a layout element such as header or footer"

   typeDescription= 'Defines a layout element such as a header or footer.'
   typeDescMsgId  = 'description_edit_layoutelement'

   meta_type = portal_type = archetype_name = "Layout Element"
   content_icon = "columndef.png"
   filter_content_types = 1
   allowed_content_types = ["HTML Block","Field Block","Link", "Person Target", "Organization Target", "Image", "ATImage"]
   global_allow = 0
   allow_discussion = 0

   schema = ATFolder.schema.copy() + Schema((
      StringField('datasource',
         default = "Action Record",
         vocabulary = "schemasVocabulary",
         widget = SelectionWidget(label="Data source"),
      ),

      StringField('layoutelement',
         default = "disabled",
         vocabulary = "layoutElementVocabulary",
         widget = SelectionWidget(label="Layout element"),
      ),

   ))

   schema["title"].mode = "r"
   schema["title"].accessor = "Title"
   schema["description"].schemata = "metadata"
   
   finalizeATCTSchema(schema)

            
   def Title(self):
      "generate title"

      # copied code here because relying on skin script did not work properly at (re)install
      from Products.CMFCore.utils import getToolByName
      # apparently, the first time this code gets run, we don't have self acquisition-wrapped?
      try:
         props = getToolByName(self, "portal_properties")["ecampaigning_properties"].element_vocabulary
      except:
         return ""
         
      data = []
      layout = "Unknown"

      k = self.getLayoutelement()

      for l in props:
         lid, lname = l.split('|')
         if lid.strip() == k:
            layout = lname
            break

      return layout


   def layoutElementVocabulary(self):
      "for selection of the element name when adding a new LED"
      vocabulary_id = "element_vocabulary"
      props = getattr(getToolByName(self,"portal_properties")["ecampaigning_properties"], vocabulary_id)

      data = []
      existing_elements = [o.getLayoutelement() for o in self.aq_parent.objectValues("Layout Element")]

      for l in props:
         lid, lname = l.split('|')
         lid = lid.strip()
         # make sure element is used in this layout, and has not been added yet
         currentelements = [e.replace("_def","") for e in self.objectIds("Layout Element")]
         if lid in currentelements and lid not in existing_elements:
            lname = lname.strip()
            data.append((lid,lname))

      data.append(self.getVocabularyItem(vocabulary_id, self.getLayoutelement()))
      data.append(("disabled","Disable Element"))
      return tuple(data)


   def addBlock(self, typename, data, condition=None, id=None):
      "add any block - a convenience for script access"
      id = id or self.generateUniqueId(typename)
      new_id = self.invokeFactory(id=id, type_name=typename)
      id = new_id or id
      block = getattr(self, id)

      if typename == "Field Block":
         schema_id = self.getDatasource()
         schema =  self.atse_getSchemaById(schema_id)
         schema_fields = [f.getName() for f in schema.fields()]
         if data in schema_fields:
            block.setFieldname(data)
         else:
            block.setCustomaccessor(data)

      elif typename == "HTML Block":
         block.setText(data)
         block.setTitle("HTML content")
      else:
         raise Exception("unsupported block type: %s" % data)

      if condition:
         block.setDisplaycondition(condition)
      
      block.reindexObject(idxs=["Title"])
      return id

         
registerType(LayoutElement)
