# AT with LinguaPlone support
try:
   from Products.LinguaPlone.public import *
except ImportError, e:
   from Products.Archetypes.public import *

from Products.ATReferenceBrowserWidget import ATReferenceBrowserWidget

#
# CAMPAIGN ACTION
#

ActionSchema = (

      StringField('name',
         schemata = "general",
         widget=StringWidget(label="Campaign action name",
           description="'Internal name' for this campaign action",
           label_msgid="label_action_name",
           description_msgid="help_action_name",
           i18n_domain="ecampaigning"
         ),
      ),
      
      IntegerField("action_count_threshold",
         default = 0,
         schemata = "advanced",
         widget=IntegerWidget(label="Minimum number of completed actions to show",
            description="The number of completed actions across <strong>all</strong> related Action Editions required before the action progress or count is displayed. This is only relevant if you plan to publicly display the live completed action progress or count.",
            label_msgid="label_action_count_threshold",
            description_msgid="help_action_count_threshold",
            i18n_domain="ecampaigning"
         )
      ),
      
      IntegerField("onsite_action_goal",
         default = 0,
         schemata = "advanced",
         widget=IntegerWidget(label="How many completed actions do you aim to achieve on THIS web site?",
            description="The number of completed actions across <strong>all</strong> related Action Editions that you are you aiming to achieve. This is only relevant if you plan to publicly display the progress or goal.",
            label_msgid="label_onsite_action_goal",
            description_msgid="help_onsite_action_goal",
            i18n_domain="ecampaigning"
         )
      ),

      StringField('publisher',
         schemata = "advanced",
         searchable=1,
         accessor="Publisher",
         widget=StringWidget(label="Organiser",
           description="The organisation, coalition or person organising the action. This is publicly listed in the Action Registry.",
           label_msgid="label_publisher",
           description_msgid="help_publisher",
           i18n_domain="ecampaigning"
         ),
      ),
)

   
#
# CAMPAIGN ACTION EDITION
#

ActionEditionSchema = (

      # DEFAULT SCHEMATA
      
      StringField('name',
         schemata = "general",
         required = True,
         widget=StringWidget(label="Action Edition Name",
                             description="The 'internal' name for this Action Edition.",
                             label_msgid="label_action_edition_name",
                             description_msgid="help_action_name",
                             i18n_domain="ecampaigning"
         ),
      ),


      StringField('title',
         schemata = "general",
         required = True,
         widget=StringWidget(label="Public Headline",
                             accessor="title",
                             description="The public headline for the Action Edition. Seen in navigation, search results, bookmarks, Action Registry, etc.",
                             label_msgid="label_action_headline",
                             description_msgid="help_action_headline",
                             i18n_domain="ecampaigning"
         ),
      ),

      StringField('actiontype',
         schemata = "general",
         required = True,
         vocabulary = "getActionTypesVocabulary",
         widget=SelectionWidget (
            label="Action Type Preset",
            description="(Re)configures the action according to predefined settings. WARNING: changing this will delete and reset all Action Steps.",
            label_msgid="label_actiontype",
            description_msgid="help_actiontype",
            i18n_domain="ecampaigning"
         ),
      ),

      ReferenceField('parentaction',
         schemata = "general",
         multiValued=False,
         allowed_types=("Campaign Action",),
         relationship="isPartOfAction",
         widget=ATReferenceBrowserWidget.ReferenceBrowserWidget(
            label="Parent Action",
            force_close_on_insert = 1,
            description="The Campaign Action this Action Edition is part of.",
            label_msgid="label_parentaction",
            description_msgid="help_parentaction",
            i18n_domain="ecampaigning"
         )
      ),
      
      
      # MESSAGE SCHEMATA
    
      ReferenceField('messagetemplate',
         schemata = "advocacy",
         required = True,
         multiValued=False,
         allowed_types=("Action Message Template",),
         relationship="usesMessageTemplate",
         widget=ATReferenceBrowserWidget.ReferenceBrowserWidget(
            label="Action Message template",
            force_close_on_insert = 1,
            description="The main message of the action: petition/pledge statement, letter/email to a target, etc.",
            label_msgid="label_message_template",
            description_msgid="help_message_template",
            i18n_domain="ecampaigning"
         )
      ),

      BooleanField("action_message_modifiable",
         default = 0,
         schemata = "advocacy",
         widget=BooleanWidget(
            label="Allow message modification",
            description="Whether or not the person taking the action can modify the message.  Mainly useful for letters/email to target(s).",
            label_msgid="label_action_message_modifiable",
            description_msgid="help_action_message_modifiable",
            i18n_domain="ecampaigning"
         )
      ),

      BooleanField("action_message_modify_as_plaintext",
         default = 1,
         schemata = "advocacy",
         widget=BooleanWidget(
            label="Convert message to plaintext for modification",
            description="Only applies if a formatted (HTML) message is used and the message is modifiable; the formatted message is then converted to plaintext so users don't have to edit HTML.",
            label_msgid="label_action_message_modify_as_plaintext",
            description_msgid="help_action_message_modify_as_plaintext",
            i18n_domain="ecampaigning"
         )
      ),
      
      ReferenceField('targets',
         schemata = "advocacy",
         required = True,
         multiValued=True,
         allowed_types=("Organization Target","Person Target"),
         relationship="targets",
         widget=ATReferenceBrowserWidget.ReferenceBrowserWidget(
            label="Targets",
            description="The individuals and/or organisations you are trying to influence.",
            label_msgid="label_action_targets", description_msgid="help_action_targets",i18n_domain="ecampaigning"
         )
      ),
      
      
      # THANK YOU - SETTINGS SCHEMATA
      
      StringField('thankyouemailname',
         required = True,
         default_method = "defaultThankyouFromName",
         schemata = "thank you",
         widget=StringWidget(
            label="Sender name for thank you emails",
            description="When a 'thank you email' is sent, this is the name from which the email will seem to have come from",
            label_msgid="label_action_thankyouemailname",
            description_msgid="help_action_thankyouemailname",
            i18n_domain="ecampaigning",
         ),
      ),

      StringField('thankyouemailfrom',
         required = True,
         default_method = "defaultThankyouFromAddress",
         validators = "isEmail",
         schemata = "thank you",
         widget=StringWidget(
            label="Sender email address for 'thank you emails'",
            description="When a 'thank you email' is sent, this is the address from which the email will seem to have come from",
            label_msgid="label_action_thankyouemailfrom",description_msgid="help_action_thankyouemailfrom",
            i18n_domain="ecampaigning",
         ),
      ),

      ReferenceField('thankyouemail',
         required=True,
         i18n_domain="ecampaigning",
         schemata = "thank you",
         multiValued=False,
         allowed_types=("Thank You Email Template",),
         relationship="usesThankyouEmailTemplate",
         widget=ATReferenceBrowserWidget.ReferenceBrowserWidget(
            force_close_on_insert = 1,
            i18n_domain="ecampaigning",
            label="'Thank you email' template",
            description="The template whose contents are used as the 'thank you email'.",
            label_msgid="label_action_thankyouemailtemplate",description_msgid="help_action_thankyouemailtemplate"
         )
      ),

      BooleanField("thankyouemail_includemessage",
         default = 1,
         schemata = "thank you",
         widget=BooleanWidget(
            label="Include a copy of the message",
            description="Whether a copy of the action message is included in the 'thank you email'.",
            label_msgid="label_action_thankyou_includecopy",
            description_msgid="help_action_thankyou_includecopy",
            i18n_domain="ecampaigning"
         )
      ),
      
      ReferenceField('thankyoupage',
         required=True,
         i18n_domain="ecampaigning",
         schemata = "thank you",
         multiValued=False,
         allowed_types=("Document", "Thank You Page Template",),
         relationship="usesThankyouPageTemplate",
         widget=ATReferenceBrowserWidget.ReferenceBrowserWidget(
            force_close_on_insert = 1,
            i18n_domain="ecampaigning",
            label="'Thank you page' template",
            description="Template whose contents are used as the 'thank you page'",
            label_msgid="label_action_thankyoupagetemplate",description_msgid="help_action_thankyoupagetemplate"
         )
      ),
      
      # ADVANCED SCHEMATA
      
      IntegerField("onsite_edition_goal",
         default = 0,
         schemata = "advanced",
         widget=IntegerWidget(label="How many completed Action Editions do you aim to achieve on THIS web site?",
                              description="The number of completed actions for <strong>this</strong> Action Edition that you are you aiming to achieve. This is only relevant if you plan to publicly display the Edition progress or goal.",
                              label_msgid="label_onsite_edition_goal",
                              description_msgid="help_onsite_edition_goal",
                              i18n_domain="ecampaigning"
         )
      ),
      
      IntegerField("edition_count_threshold",
         default = 0,
         schemata = "advanced",
         widget=IntegerWidget(
            label="Minimum number of completed actions to show",
            description="The number of completed actions for <strong>this</strong> Action Edition required before the action progress or count is displayed. This is only relevant if you plan to publicly display the live Edition action progress or count.",
            label_msgid="label_edition_count_threshold",
            description_msgid="help_edition_count_threshold",
            i18n_domain="ecampaigning"
         )
      ),

      LinesField('coverage',
         schemata = "advanced",
         widget = InAndOutWidget(label="Action Edition Coverage",
                                 description="The geographic areas the Action Edition is relevent to. Used to help promote the Action Edition via the Action Registry.",
                                 label_msgid="label_action_coverage",
                                 description_msgid="help_action_coverage",
                                 i18n_domain="ecampaigning"
         ),
         vocabulary = "localizedCountryVocabulary",
      ),

      StringField('publisher',
         schemata = "advanced",
         searchable=1,
         accessor="Publisher",
         widget=StringWidget(label="Organiser",
           description="The organisation, coalition or person organising the action.  This is publicly listed in the Action Registry.",
           label_msgid="label_publisher",
           description_msgid="help_publisher",
           i18n_domain="ecampaigning"
         ),
      ),
)

#
# CAMPAIGN ACTION RECORD (SIGNUPS)
#

ActionRecordSchema = (

      StringField('givenname', schemata = "personal", required = True,
         widget=StringWidget(label="Given name",
            label_msgid="label_givenname",
            description_msgid="help_givenname",
            i18n_domain="ecampaigning"
         )
      ),

      StringField('familyname', schemata = "personal", required = True,
         widget=StringWidget(label="Family name(s)",
            label_msgid="label_familyname",
            description_msgid="help_familyname",
            i18n_domain="ecampaigning"
         )
      ),

      StringField('emailaddress',
         required = True,
         schemata = "personal",
         widget=StringWidget(label="Email address",
            description="",
            label_msgid="label_emailaddress",
            description_msgid="help_emailaddress",
            i18n_domain="ecampaigning"
         ),
         validators = "isEmail"
      ),

      StringField('streetaddress',
         required = True,
         schemata = "location",
         widget=LinesWidget(label="Street address",
            label_msgid="label_streetaddress",
            description_msgid="help_streetaddress",
            i18n_domain="ecampaigning",
            rows=2
         ),
      ),

      StringField('postalcode',
         required = True,
         schemata = "location",
         widget=StringWidget(label="Postal code",
            label_msgid="label_postalcode",
            description_msgid="help_postalcode",
            i18n_domain="ecampaigning"
         ),
      ),

      StringField('city',
         required = True,
         schemata = "location",
         widget=StringWidget(label="City",
            label_msgid="label_city",
            description_msgid="help_city",
            i18n_domain="ecampaigning"
         ),
      ),

      StringField('country',
         required = True,
         schemata = "location",
         vocabulary = "localizedCountryVocabulary",
         widget=SelectionWidget(label="Country",
            label_msgid="label_country",
            description_msgid="help_country",
            i18n_domain="ecampaigning"
         ),
      ),

      BooleanField('anonymous',
         schemata = "personal",
         default=0,
         widget=BooleanWidget(label="Keep me anonymous",
            description="Do not publicly show my name, country and message.",
            label_msgid="label_anonymous",
            description_msgid="help_anonymous",
            i18n_domain="ecampaigning"
         ),
      ),

      BooleanField('receive_updates',
         schemata = "personal",
         default=0,
         widget=BooleanWidget(label="Keep me updated.",
            description="Select this box to be kept informed of new actions and updates on how the campaign progresses (see <a href='/PrivacyPolicy'>privacy policy</a>)",
            label_msgid="label_receive_updates",
            description_msgid="help_receive_updates",
            i18n_domain="ecampaigning"
         ),
      ),

      ComputedField('fullname', 
         schemata = "automatic",
         expression="here._fullName()",
      ),

      ComputedField('title',
         schemata = "automatic",
         accessor = "Title",
         expression="here._fullName()",
      ),

      # This is a decoy field that is made invisible via CSS but that form spam scripts will complete.
      # If completed, it was either submitted by a form spam script, a user with accessibility needs or a non CSS browser (rare)
      # The label and description are meant to be used if a user with accessibility needs views the form.
      # If this field name is changed, the corresponding CSS needs to also change to ensure it stays invisible
      StringField('name_df', schemata = "automatic",
         widget=StringWidget(label="Feedback",
            description="Do you have any feedback for improving the accessibility of this action?",
            label_msgid="label_name_df",
            description_msgid="help_name_df",
            i18n_domain="ecampaigning"
         )
      ),

      StringField("messagesubject",
         default_method = "getMessageSubject",
         schemata = "message",
         widget=StringWidget(label="Message subject",
            description="This line will be the subject / title of your message",
            label_msgid="label_messagesubject",
            description_msgid="help_messagesubject",
            i18n_domain="ecampaigning"
         ),
      ),

      TextField('message',
         default_method = "getMessageText",
         allowable_content_types = ("text/plain", "text/html"),
         schemata = "message",
         default_content_type="text/html",
         default_output_type="text/html",
         widget=TextAreaWidget(label="Message body",
            description="Salutation/greetings will be added automatically.",
            label_msgid="label_message",
            description_msgid="help_message",
            i18n_domain="ecampaigning",
            rows=15
         )
      ),
)


#
# CLIENT TRACKING
#

ClientInfoSchema = (

      StringField("ip_address",
         schemata = "clientinfo",
         widget=StringWidget(label="Supporter IP address",
            description="The IP address of the supporter who took the action",
            label_msgid="label_supporter_ip_address",
            description_msgid="help_supporter_ip_address",
            i18n_domain="ecampaigning",
            visible={'edit':'invisible', 'view':'visible'}
         ),
      ),

      StringField("user_agent",
         schemata = "clientinfo",
         widget=StringWidget(label="User agent",
            description="Supporter WWW user agent string",
            label_msgid="label_user_agent",
            description_msgid="help_user_agent",
            i18n_domain="ecampaigning",
            visible={'edit':'invisible', 'view':'visible'}
         ),
      ),

      StringField("user_agent_language",
         schemata = "clientinfo",
         widget=StringWidget(label="User agent language",
            description="Supporter WWW user agent language string",
            label_msgid="label_user_agent_language",
            description_msgid="help_user_agent_language",
            i18n_domain="ecampaigning",
            visible={'edit':'invisible', 'view':'visible'}
         ),
      ),

)


#
# ORIGIN TRACKING
#

OriginInfoSchema = (


   StringField("origin_url",
      schemata = "origininfo",
      widget=StringWidget(label="Origin URL",
         description="The URL from which the supporter came to the site",
         label_msgid="label_origin_url",
         description_msgid="help_origin_url",
         i18n_domain="ecampaigning",
         visible={'edit':'invisible', 'view':'visible'}
      ),
   ),

   StringField("onsite_origin_url",
      schemata = "origininfo",
      widget=StringWidget(label="On-site origin URL",
         description="The URL the supporter first visited on the site",
         label_msgid="label_onsite_origin_url",
         description_msgid="help_onsite_origin_url",
         i18n_domain="ecampaigning",
         visible={'edit':'invisible', 'view':'visible'}
      ),
   ),

   StringField("entry_url",
      schemata = "origininfo",
      widget=StringWidget(label="Entry URL",
         description="The URL which the supporter first visited on the site",
         label_msgid="label_entry_url",
         description_msgid="help_entry_url",
         i18n_domain="ecampaigning",
         visible={'edit':'invisible', 'view':'visible'}
      ),
   ),

   ComputedField("origin_id",
      expression = "here.mapOriginURLParameters(here._getEntryURLParameters())['origin']",
      widget=StringWidget(label="Origin identifier",
         description="Origin type and/or organization identifier",
         visible={'edit':'invisible', 'view':'visible'}
      ),
      schemata = "origininfo"
   ),

   ComputedField("origin_person_id",
      expression = "here.mapOriginURLParameters(here._getEntryURLParameters())['person']",
      widget=StringWidget(label="Personal identifier",
         description="For example in the case of HTML email",
         visible={'edit':'invisible', 'view':'visible'}
      ),
      schemata = "origininfo"
   ),

   ComputedField("origin_version_id",
      expression = "here.mapOriginURLParameters(here._getEntryURLParameters())['version']",
      widget=StringWidget(label="Version code",
         description="For example if different banner versions",
         visible={'edit':'invisible', 'view':'visible'}
      ),
      schemata = "origininfo"
   ),

   ComputedField("origin_segment_id",
      expression = "here.mapOriginURLParameters(here._getEntryURLParameters())['segment']",
      widget=StringWidget(label="Segment",
         description="Supporter segment, for targeted promotions",
         visible={'edit':'invisible', 'view':'visible'}
      ),
      schemata = "origininfo"
   ),

   ComputedField("origin_keywords",
      expression = "here.mapOriginURLParameters(here._getEntryURLParameters())['keywords']",
      widget=StringWidget(label="Keywords",
         description="Keyword(s) something was promoted with, separated by commas",
         visible={'edit':'invisible', 'view':'visible'}
      ),
      schemata = "origininfo"
   ),

)

#
# PERSON
#

PersonSchema = (

      StringField("prefix",
         widget=StringWidget(label="Name prefix"),
         schemata = "default"
      ),

      StringField("givenname",
         widget=StringWidget(label="Given name"),
         required = True,
         schemata = "default"
      ),

      StringField("middlename",
         widget=StringWidget(label="Middle name(s)"),
         schemata = "default"
      ),

      StringField("familyname",
         widget=StringWidget(label="Family name"),
         required = True,
         schemata = "default"
      ),

      StringField("suffix",
         widget=StringWidget(label="Name suffix"),
         schemata = "default"
      ),

      StringField("position",
         widget=StringWidget(label="Position", description="Position or role in the organization"),
         schemata = "default"
      ),

      StringField("emailaddress",
         widget=StringWidget(label="Email address"),
         validators = "isEmail",
         schemata = "default"
      ),

      StringField("phonenumber",
         widget=StringWidget(label="Phone number"),
         schemata = "default"
      ),

      StringField("faxnumber",
         widget=StringWidget(label="Fax number"),
         schemata = "default"
      ),

      StringField("streetaddress",
         widget=LinesWidget(label="Street address", rows=2),
         schemata = "default"
      ),

      StringField('city',
         widget=StringWidget(label="City"),
         schemata = "default"
      ),

      StringField('postalcode',
         widget=StringWidget(label="Postal code"),
         schemata = "default"
      ),

      StringField("country",
         vocabulary = "localizedCountryVocabulary",
         widget=SelectionWidget(label="Country"),
         schemata = "default"
      ),

      ComputedField("title",
         accessor = "Title",
         expression = "here.buildTitle()",
         widget=StringWidget(view={"edit":"invisible"})
      )

)

#
# ORGANIZATION
#  

OrganizationSchema = (

      StringField("title",
         accessor="Title",
         widget=StringWidget(label="Organisation Name"),
         required = True,
         schemata = "default"
      ),

      StringField("phonenumber",
         widget=StringWidget(label="Phone number"),
         schemata = "default"
      ),

      StringField("faxnumber",
         widget=StringWidget(label="Fax number"),
         schemata = "default"
      ),

      StringField("emailaddress",
         widget=StringWidget(label="Email address"),
         validators = "isEmail",
         schemata = "default"
      ),

      StringField("streetaddress",
         widget=LinesWidget(label="Street address", rows=2),
         schemata = "default"
      ),

      StringField('city',
         widget=StringWidget(label="City"),
         schemata = "default"
      ),

      StringField('postalcode',
         widget=StringWidget(label="Postal code"),
         schemata = "default"
      ),

      StringField("country",
         vocabulary = "localizedCountryVocabulary",
         widget=SelectionWidget(label="Country"),
         schemata = "default"
      ),

      StringField("wwwsite",
         widget=StringWidget(label="Organisation's web site address", description="Include 'http://' at the beginning."),
         validators = "isURL",
         schemata = "default"
      ),

)
