# AT with LinguaPlone support
try:
   from Products.LinguaPlone.public import *
except ImportError, e:
   from Products.Archetypes.public import *

#ATCT
try:
   from Products.ATContentTypes.content.document import ATDocument
   from Products.ATContentTypes.content.schemata import finalizeATCTSchema
# old ATCT
except ImportError:
   from Products.ATContentTypes.types.ATDocument import ATDocument
   finalizeATCTSchema = lambda x: None

from Products.ATDataMerge.content import DataMergeStringField, DataMergeTextField
from Products.eCampaigning.util import getDefaultMergeData

__all__ = ["ThankyouPageTemplate","ThankyouEmailTemplate","MessageTemplate"]

class DataMergeTemplate(ATDocument):
   "template base class"

   global_allow = 0
   _at_rename_after_creation = True

   schema = ATDocument.schema.copy() + Schema((

      DataMergeStringField('messagetitle',
         required=True,
         searchable=True,
         widget=StringWidget(label="Message title",
                label_msgid="label_messagesubject",
                description="The message subject or title",
                description_msgid="help_messagesubject",
                i18n_domain="ecampaigning"
         ),
      ),
    
      DataMergeTextField("text",
        required=True,
        searchable=True,
        primary=True,
        default_content_type = 'text/html',
        default_output_type = 'text/x-html-captioned',
        allowable_content_types = ("text/plain", "text/html"),
        widget=RichWidget(
            description = "The message text.",
            description_msgid = "help_message_text",
            label = "Message text",
            label_msgid = "label_message_text",
            rows = 25,
            i18n_domain = "ecampaigning",
            allow_file_upload = True
        )
      )
   ))
   
   schema["description"].schemata = "metadata"

   try:
      schema.moveField("messagetitle", after="title")
   except AttributeError:
      from Products.eCampaigning.util import moveFieldInSchema
      moveFieldInSchema(schema, "messagetitle", 1)

   finalizeATCTSchema(schema)

   def compileMergeData(self, customcontext):
      "provide default fields & values for data merge, plus a hook for customizations ('getMergeData')"
      
      context = customcontext or self

      # try & get the record & set the context
      try:
         context = self.restrictedTraverse(self.REQUEST.SESSION["record_path"])
      except:
         pass

      return getDefaultMergeData(context)
      

class MessageTemplate(DataMergeTemplate):
   "message template"

   meta_type = portal_type = archetype_name = "Action Message Template"

   schema = DataMergeTemplate.schema.copy()
   tw = schema["text"].widget
   tw.description = "The action message text shown on the action-taking page"
   tw.description_msgid = "help_field_action_message"
   tw.label = "Action message"
   tw.label_msgid = "label_field_action_message"

   finalizeATCTSchema(schema)

registerType(MessageTemplate)


class ThankyouPageTemplate(DataMergeTemplate):
   "thank you page"

   meta_type = portal_type = archetype_name = "Thank You Page Template"

   schema = DataMergeTemplate.schema.copy()
   tw = schema["text"].widget
   tw.description = "The text content of the page to be shown after the action has been taken"
   tw.description_msgid = "help_field_thankyoupage_message"
   tw.label = "Thank-you message"
   tw.label_msgid = "label_field_thankyoupage_message"

   finalizeATCTSchema(schema)

registerType(ThankyouPageTemplate)


class ThankyouEmailTemplate(DataMergeTemplate):
   "thank you email"

   meta_type = portal_type = archetype_name = "Thank You Email Template"

   schema = DataMergeTemplate.schema.copy()
   tw = schema["text"].widget
   tw.description = "The email message to be sent after the action has been taken"
   tw.description_msgid = "help_field_thankyouemail_message"
   tw.label = "Thank-you email message"
   tw.label_msgid = "label_field_thankyouemail_message"

   finalizeATCTSchema(schema)


registerType(ThankyouEmailTemplate)
