# python
import re
from email.Header import Header

# zope
from AccessControl import ModuleSecurityInfo
from Products.PageTemplates.Expressions import getEngine
from Acquisition import ImplicitAcquisitionWrapper
from Products.PythonScripts.standard import url_unquote

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.utils import mapply
from Products.Archetypes.Field import *

from Products.Archetypes.public import SelectionWidget, MultiSelectionWidget

# package
from content.record import ActionRecord
from config import CUSTOM_DATAMERGE_SCRIPT

_marker = []

ModuleSecurityInfo("Products.eCampaigning.util").declarePublic("expand_relative_urls", "persistActionFormContents", "process_tracking_variables")


def expand_relative_urls(text, base):
   "replace relative urls with absolute ones"
   ValidURISchemes = 'http|https|shttp|mailto|ftp|sftp|news|nntp|rtsp|mms|cid|xmpp|callto|skype|ymsgr|msnim|aim|irc|ircs|cvs|feed|webcal'
   baseadder = lambda m: base + m.group(0)
   return re.sub('(?<=href=")(?!' + ValidURISchemes + ')([^"]+)', baseadder, text)


def evaluateTales(context, expressionstring):
   "evaluate TALES expression string in context"
   compiled = getEngine().compile(expressionstring)
   result = compiled(context)
   if isinstance(result, Exception):
      raise result
   return result


#TODO: check if expression caching should be added
def getActionExpressionContext(context):
   "get TALES expression context for use in action step sequence"
   ptool = getToolByName(context, 'portal_url')
   portal = ptool.getPortalObject()
   request = context.REQUEST
   session = request.SESSION
   edition = context.getRecordContainer().aq_parent
   action = edition.getParentaction()

   try:
      record = context.restrictedTraverse(session["record_path"])
   except:
      record = None

   data = {
      "here": context,
      "portal": portal,
      "action": action,
      "edition": edition,
      "record": record,
      "request": request,
      "session": session
   }

   return getEngine().getContext(data)



def persistActionFormContents(context, form, record):
   "store contents"

   for field in context.atse_getSchemaById("Action Record").fields():
      widget = field.widget
      result = widget.process_form(context, field, form, empty_marker=_marker)

      if result is _marker or result is None:
         continue

      result = result[0]

      fname = field.getName()

      # if the message is not modifiable or the user has not modified the message, do not save it
      if fname == "message":
         if (not context.getAction_message_modifiable()) or (result.strip() == context.getMessagetemplate().getText().strip()):
            continue

      # MAP (AT) FIELD TYPES TO (ZOPE) PROPERTY TYPES
      typemap={
         "StringField": ("string", lambda x: x),
         "TextField": ("text", lambda x: x),
         "BooleanField": ("boolean", lambda x: bool(eval(x))),
         "FloatField": ("float", lambda x: x),
         "IntegerField": ("long", lambda x: x),
         "DateTimeField": ("date", lambda x: x)
      }

      ftype = field.getType().split('.')[-1]

      try:
         ptype, vfunction = typemap[ftype]
      except:
         raise Exception("unsupported record field type:%s (value:%.20s)" % (ftype, result))

      if record.hasProperty(fname):
         record._setPropValue(fname, vfunction(result))
      else:
         record._setProperty(fname, vfunction(result), ptype)


# modified from Archetypes.BaseObject.validate & Schema.validate
def validate_fields(context, fields_to_validate, errors):
   "Validates the form data from the request"

   form = context.REQUEST.form
   instance = context.aq_inner

   fields = [f for f in context.atse_getSchemaById("Action Record").fields()]
   fields = [f for f in fields if f.getName() in fields_to_validate]
   fields = [f for f in fields if f.getName() not in errors]

   if fields == []:
      raise Exception("no fields to validate!")

   for field in fields:
      widget = field.widget
      name = field.getName()

      # parse the form
      parse_result = widget.process_form(instance, field, form, empty_marker=_marker)
      if parse_result is _marker or parse_result is None:
         # radio buttons without default value are not passed by browser, so fake empty
         # two widgets covered - add more as found necessary
         if isinstance(widget, (SelectionWidget, MultiSelectionWidget)):
            parse_result=[None]
         else:
            formtxt = str(form.items()) # ["<p>'%s' - '%s'</p>" % (str(key), str(value)[:20]) for key, value in form.items()]
            raise Exception("form validation for field %s failed:\n%s" % (name, formtxt))
      value = parse_result[0]

      # validate the field; if it already has an error, don't add
      validation_result = field.validate(instance=instance, value=value, REQUEST=context.REQUEST, errors=errors)
      if validation_result and name not in errors:
         errors[name] = validation_result

   return errors


def moveFieldInSchema(schema, fieldName, index, schemata=None):
    """Given a Schema and the name of a field in this schema, move the field
    within its schemata to the position given by index. For example, giving
    index 0 will move the field to the top of the schema. Negative indexes
    are also permitted: Giving index -1 will move it to the bottom, whilst
    -2 will make the field the second-last in its schemata.

    If the 'schemata' argument is given, this gives the name of the schemata
    the field should be placed in. If the field is in another schemata, it will
    be moved.

    The method will *not* create a copy of schema before modifying it. If you
    are moving fields from a shared schema (e.g. BaseSchema or BaseMetadata),
    you should create a copy before passing the copy in to this method, to
    avoid modifying *all* instances of that schema. Since the schema is modified
    in-place, the method does not return anything.

    May raise KeyError if fieldName cannot be found, IndexError if the index
    is out of range.
    """

    field = schema[fieldName]

    if schemata is not None:
        field.schemata = schemata
    else:
        schemata = field.schemata

    # Code adapted from ManagedSchema.moveField()

    fields = schema.fields()
    fieldNames = [f.getName() for f in fields]
    schemataNames = schema.getSchemataNames()

    schemataFields = {}
    for schemataName in schemataNames:
        schemataFields[schemataName] = schema.getSchemataFields(schemataName)

    currentSchemataFields = schemataFields[schemata]
    currentIndex = [f.getName() for f in currentSchemataFields].index(field.getName())

    if index < 0:
        index = len(currentSchemataFields) - (-1 * index)

    # Move the item by deleting it and re-inserting it in the schemata list
    del currentSchemataFields[currentIndex]
    currentSchemataFields.insert(index, field)

    schemataFields[schemata] = currentSchemataFields

    # Re-initialise the schema
    schema.__init__()
    for schemataName in schemataNames:
        for f in schemataFields[schemataName]:
            schema.addField(f)


def decode_encodedvocabulary(vocabularydata, id):
   "get list of data items stored in newline-separated title|id:data1+data2+data3... lines, by id"
   data = [l for l in vocabularydata if id in l]
   if data:
      return data[0].strip().split('|')[1].split(':')[1].split('+')
   return ()


def encode_smtp_header(s,encoding="utf-8"):
   "utf encoding support"
   return str(Header(s, encoding))


def getDefaultMergeData(context):
   "populate data merge namespace with global defaults"

   # context should be the record, if available

   data = {}

   # ONSITE Action Edition Level Data
   try:
       onsite_edition_count = context.getRecordCount()
   except Exception, e:
       onsite_edition_count = "[could not retrieve # of edition taken: %s]" % e

   try:
       onsite_edition_goal = context.getOnsite_edition_goal()
   except Exception, e:
       onsite_edition_goal = "[could not retrieve value of the on-site edition goal: %s]" % e

   try:
       onsite_edition_progress = int((float(onsite_edition_count) / onsite_edition_goal) * 100 + 0.5)
   except Exception, e:
       onsite_edition_progress = "[could not calculate the value of edition progress: %s]" % e

   # ONSITE Campaign Action Level Data
   try:
       onsite_action_count = context.getRecordCount()
   except Exception, e:
       onsite_action_count = "[could not retrieve # of action taken: %s]" % e

   try:
       onsite_action_goal = context.getOnsite_action_goal()
   except Exception, e:
       onsite_action_goal = "[could not retrieve value of the on-site action goal: %s]" % e

   try:
       onsite_action_progress = int((float(onsite_action_count) / onsite_action_goal) * 100 + 0.5)
   except Exception, e:
       onsite_action_progress = "[could not calculate the value of action progress: %s]" % e

   # OVERALL Campaign Action Data (onsite + offsite)
   # TO BE IMPLEMENTED LATER

   # AT Data Merge Data
   if hasattr(context, CUSTOM_DATAMERGE_SCRIPT):
      try:
         data.update(getattr(context,CUSTOM_DATAMERGE_SCRIPT)(context))
         status = "[queried custom data provider]"
      except Exception, e:
         status = "[error in custom data provider: %s]" % e
   else:
      status = "[no custom data provider found (context: %s)]" % context.getId()

   data.update({
      "__ec_dm": "__ec_dm_success",
      "onsite_edition_count":onsite_edition_count,
      "onsite_edition_goal":onsite_edition_goal,
      "onsite_edition_progress":onsite_edition_progress,
      "onsite_action_count":onsite_action_count,
      "onsite_action_goal":onsite_action_goal,
      "onsite_action_progress":onsite_action_progress,
      "status_custom_data_provider": status
   })

   return data


def process_tracking_variables(url, mapping, terms="tracked_terms"):
   """query str vars from entry_url into a dict,
      keys mapped to longer names from mapping
   """

   queryvars =  url.split('?')[1].split('&')
   pairs = dict([pair.split('=') for pair in queryvars])
   parsed = [(mapping[name], url_unquote(pairs[name])) for name in pairs if name in mapping]
   vars = dict.fromkeys(mapping.values())
   vars.update(dict(parsed))

   try: # special case: the '+' separated keywords, if the terms var was submitted in qs
      vars[terms] = vars[terms].split('+')
   except:
      vars[terms] = ()

   return vars
