from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.CMFCore.WorkflowTool import addWorkflowFactory

from config import PROJECTNAME

WORKFLOW_ID = "action_workflow"
WOFKFLOW_TITLE = "Action Workflow [%s]" % PROJECTNAME

def setupAction_workflow(wf):
    "..."
    
    # title
    wf.setProperties(title='Action Workflow [eCampaigning]')

    # states
    for s in ['private', 'staged', 'pending', 'approved', 'published', 'closed']:
        wf.states.addState(s)
        
    # transitions
    for t in ['hide', 'stage', 'submit', 'reject', 'retract', 'approve', 'launch', 'close']:
        wf.transitions.addTransition(t)
        
    # variables
    for v in []:
        wf.variables.addVariable(v)
        
    # worklists
    for l in []:
        wf.worklists.addWorklist(l)
   
    # permissions managed by workflow
    for p in ('Access contents information', 'Modify portal content', 'View'):
        wf.addManagedPermission(p)


    # Initial State
    wf.states.setInitialState('private')

    # States initialization
    
    sdef = wf.states['private']
    sdef.setProperties(title = """Private""", transitions=('stage',))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 0, ['Manager', 'Owner'])

    # state definition
    sdef = wf.states['staged']
    sdef.setProperties(
      title = """Being staged""",
      transitions = ('hide', 'submit')
    )
    sdef.setPermission('Access contents information', 0, ['Member', 'Manager', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 0, ['Member', 'Manager', 'Reviewer'])

    # state definition
    sdef = wf.states['pending']
    sdef.setProperties(
      title = """Submitted for review""",
      transitions = ('approve', 'reject', 'retract')
    )
    sdef.setPermission('Access contents information', 0, ['Member', 'Manager', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager','Reviewer'])
    sdef.setPermission('View', 0, ['Member', 'Manager', 'Reviewer', 'Anonymous'])

    # state definition
    sdef = wf.states['approved']
    sdef.setProperties(
      title="""Ready to launch""",
      transitions=('retract', 'reject', 'launch')
    )
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner', 'Reviewer', 'Member'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 0, ['Manager', 'Owner', 'Reviewer', 'Member', 'Anonymous'])
    
    # state definition
    sdef = wf.states['published']
    sdef.setProperties(
      title="""Live""",
      transitions=('retract', 'close')
    )
    sdef.setPermission('Access contents information', 0, ['Anonymous', 'Member', 'Manager', 'Reviewer','Owner'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 0, ['Anonymous', 'Member', 'Manager', 'Member', 'Owner'])

    # state definition
    sdef = wf.states['closed']
    sdef.setProperties(
      title="""Closed""",
      transitions=('hide',)
    )
    sdef.setPermission('Access contents information', 0, ['Member', 'Manager', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 0, ['Member', 'Manager', 'Reviewer'])

    ## Transitions initialization
    
    tdef = wf.transitions['hide']
    tdef.setProperties(
        title="""Member makes content private""",
        new_state_id="""private""",
        trigger_type=1,
        script_name="""""",
        after_script_name="""""",
        actbox_name="""Make private""",
        actbox_url="""%(content_url)s/content_hide_form""",
        actbox_category="""workflow""",
        props=None,
    )

    tdef = wf.transitions['stage']
    tdef.setProperties(
        title="""Member stages the action""",
        new_state_id="""staged""",
        trigger_type=1,
        script_name="""""",
        after_script_name="""""",
        actbox_name="""Stage""",
        actbox_url="""%(content_url)s/content_show_form""",
        actbox_category="""workflow""",
        props={'guard_roles': 'Owner;Manager'},
    )

    tdef = wf.transitions['submit']
    tdef.setProperties(
        title="""Owner submits the content for approval""",
        new_state_id="""pending""",
        trigger_type=1,
        script_name="""""",
        after_script_name="""""",
        actbox_name="""Submit for approval""",
        actbox_url="""%(content_url)s/content_submit_form""",
        actbox_category="""workflow""",
        props={'guard_roles': 'Owner;Manager'},
    )

    tdef = wf.transitions['reject']
    tdef.setProperties(
        title="""Reviewer returns the action to be re-staged""",
        new_state_id="""staged""",
        trigger_type=1,
        script_name="""""",
        after_script_name="""""",
        actbox_name="""Reject""",
        actbox_url="""%(content_url)s/content_reject_form""",
        actbox_category="""workflow""",
        props={'guard_roles': 'Reviewer;Manager'},
    )

    tdef = wf.transitions['approve']
    tdef.setProperties(
        title="""Reviewer approves the action for launch""",
        new_state_id="""approved""",
        trigger_type=1,
        script_name="""""",
        after_script_name="""""",
        actbox_name="""Approve for launch""",
        actbox_url="""%(content_url)s/content_show_form""",
        actbox_category="""workflow""",
        props={'guard_roles': 'Reviewer;Manager'},
    )
    
    tdef = wf.transitions['retract']
    tdef.setProperties(
        title="""Owner or Reviewer retracts open action""",
        new_state_id="""staged""",
        trigger_type=1,
        script_name="""""",
        after_script_name="""""",
        actbox_name="""Re-stage the action""",
        actbox_url="""%(content_url)s/content_retract_form""",
        actbox_category="""workflow""",
        props={'guard_roles': 'Owner;Reviewer;Manager'},
    )
    
    tdef = wf.transitions['launch']
    tdef.setProperties(
        title="""Owner or Reviewer launches action""",
        new_state_id="""published""",
        trigger_type=1,
        script_name="""""",
        after_script_name="""""",
        actbox_name="""Launch""",
        actbox_url="""%(content_url)s/content_publish_form""",
        actbox_category="""workflow""",
        props={'guard_roles': 'Owner;Reviewer;Manager'},
    )

    tdef = wf.transitions['close']
    tdef.setProperties(
        title="""Owner or Reviewer closes action""",
        new_state_id="""closed""",
        trigger_type=1,
        script_name="""""",
        after_script_name="""""",
        actbox_name="""Close""",
        actbox_url="""%(content_url)s/content_hide_form""",
        actbox_category="""workflow""",
        props={'guard_roles': 'Owner;Reviewer;Manager'},
    )
    
    ## State Variable
    wf.variables.setStateVar('review_state')

    ## Variables initialization
    ## Worklists Initialization

def createAction_workflow(id):
    "..."
    ob = DCWorkflowDefinition(id)
    setupAction_workflow(ob)
    return ob

addWorkflowFactory(createAction_workflow, id=WORKFLOW_ID)
